/*
 * Copyright (C) 2001 USAGI/WIDE Project.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the project nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * This is a parse routine for a message of SADB_DELETE.
 */
#ifdef PFKEY_DEBUG
#define DEBUG 1
#endif

#ifdef MODULE
#include <linux/module.h>
#  ifdef MODVERSIONS
#  include <linux/modversions.h>
#  endif
#endif

#include <linux/config.h>
#include <linux/version.h>
#include <linux/kernel.h>
#include <linux/malloc.h>
#include <linux/socket.h>
#include <linux/in.h>
#include <linux/in6.h>
#include <linux/random.h>

#include <net/pfkeyv2.h>
#include <net/sadb.h>

#include "pfkey_v2_msg.h"
#include "sadb_utils.h" /* esptoa authtoa */
			/* sockaddrtoa ultoa */
#define BUFSIZE 64

int sadb_msg_delete_parse(struct sock *sk, struct sadb_msg *msg, struct sadb_msg **reply)
{
	int error = 0;
	__u64 ext_exist = 0;
	struct sadb_ext *ext_msgs[SADB_EXT_MAX+1];
	struct sadb_sa  *sa;
	struct sadb_address *src;
	struct sadb_address *dst;
	struct sockaddr_storage saddr, daddr;

	if(!msg){
		pr_debug("sadb_msg_delete_parse: msg==null\n");
		error = -EINVAL;
		goto err;
	}

	error = sadb_msg_detect_ext(msg, ext_msgs, &ext_exist);
	if(error){
		pr_debug("sadb_msg_delete_parse: error in sadb_msg_detect_ext\n");
		goto err;
	}

	if(ext_exist&(1<<SADB_EXT_SA) &&
		ext_exist&(1<<SADB_EXT_ADDRESS_SRC) &&
		ext_exist&(1<<SADB_EXT_ADDRESS_DST))
	{
		sa  = (struct sadb_sa*)ext_msgs[SADB_EXT_SA];

		src = (struct sadb_address*)ext_msgs[SADB_EXT_ADDRESS_SRC];
		dst = (struct sadb_address*)ext_msgs[SADB_EXT_ADDRESS_DST];

		memset(&saddr, 0, sizeof(struct sockaddr_storage));
		memset(&daddr, 0, sizeof(struct sockaddr_storage));

		error = sadb_address_to_sockaddr(src, (struct sockaddr*)&saddr);
		if(error){
			pr_debug("sadb_msg_delete_parse: src translate\n");
			goto err;
		}
		error = sadb_address_to_sockaddr(dst, (struct sockaddr*)&daddr);
		if(error){
			pr_debug("sadb_msg_delete_parse: dst translate\n");
			goto err;
		}
		error = sadb_remove((struct sockaddr*)&saddr,
			    (struct sockaddr*)&daddr,
			    sa->sadb_sa_spi);
		if(error){
			pr_debug("sadb_msg_delete_parse: sadb_remove failed: error=%d\n", error);
			goto err;
		}
	}

//	error = sadb_msg_reply_build(msg, ext_msgs, ext_exist, reply);
	if(error)
		goto err;
	return error;
err:
//	sadb_msg_reply_build2(msg, error, reply);
	return error;
}


